#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <Wire.h>
#include "Adafruit_HDC1000.h"
#include <time.h>
#include <WiFiUdp.h>

const char* PI_SSID = "Pi3-AP";
const char* PI_PASSWORD = "raspberry";
const int   MQTT_PORT = 1883;
const char* MQTT_SERVER = "10.0.0.254";
const char* MQTT_INIT_TOPIC = "/dev/init/msg/";
const char* MQTT_SN_TOPIC = "/Raspi3/hdc1000";
const char* MQTT_INIT_TOPIC_MSG = "{\"dev\"=\"hdc1000\",\"msg\"=\"init\"}";

// HDC1000 Setting
Adafruit_HDC1000 hdc = Adafruit_HDC1000();

// NTP Setting
//WiFiUDP ntpUDP;
const char *NTP_SERVER = "ntp.nict.jp";
const int NTP_TIME_OFFSET = 9 * 60 * 60;  // JST（+9h）

WiFiClient espClient;
PubSubClient client(espClient);

//WiFi Setting
void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(PI_SSID);

  WiFi.begin(PI_SSID, PI_PASSWORD);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("retry..");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected !");
  Serial.println("esp8266 IP address: ");
  Serial.println(WiFi.localIP());
}


// MQTT Connect
void mqttConnect() {
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    String clientId = "ESP8266-";  // Create client ID
    clientId += String(random(0xffff), HEX);

    if (client.connect(clientId.c_str())) {
      Serial.println("MQTT connected");
      client.publish(MQTT_INIT_TOPIC, MQTT_INIT_TOPIC_MSG);
    } else {
      Serial.print("Conecct failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }

  }
}

void setup() {
  Serial.begin(115200);
  setup_wifi();
  client.setServer(MQTT_SERVER, MQTT_PORT);
  if (!hdc.begin()) {
    Serial.println("Couldn't find sensor!");
    while (1);
  }

  configTime( NTP_TIME_OFFSET, 0, NTP_SERVER);
}

void loop() {
  time_t t = time(NULL);
  struct tm *tm;
  tm = localtime(&t);
  char dt[25];
  sprintf(dt, "%04d-%02d-%02dT%02d:%02d:%02d+09:00",
    tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
    tm->tm_hour, tm->tm_min, tm->tm_sec);
   Serial.printf(dt);

  if (!client.connected()) {
    mqttConnect();
  } else {
    client.loop();
  }
  char pub_json[100];
  
  char temp_buf[4];
  char humi_buf[4];
  float temp  = hdc.readTemperature();
  float humi  = hdc.readHumidity();
  
  sprintf(pub_json,"{\"temperature\":%s, \"humidity\": %s, \"@timestamp\": %s}]}",
          dtostrf(temp, 4, 2, temp_buf),dtostrf(humi, 4, 2, humi_buf),dt);
  client.publish(MQTT_SN_TOPIC, pub_json);
  Serial.println("Temperature Publish!");
  Serial.println(pub_json);
  
  delay(60000 * 10);
  //delay(5000);
}

