#include <ESP8266WiFi.h>
#include <PubSubClient.h>

const char* PI_SSID = "Pi3-AP";
const char* PI_PASSWORD = "raspberry";
const int   MQTT_PORT = 1883;
const char* MQTT_SERVER = "10.0.0.254";
const char* MQTT_INIT_TOPIC = "/dev/init/msg/";
const char* MQTT_LED_TOPIC = "/esp8266/led/line";
const char* MQTT_INIT_TOPIC_MSG = "{\"dev\"=\"led\",\"msg\"=\"init\"}";

// Pin Setting
const int ledPin = 4;    // 人感センサーpin（定数）

WiFiClient espClient;
PubSubClient client(espClient);

//WiFi Setting
void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(PI_SSID);

  WiFi.begin(PI_SSID, PI_PASSWORD);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("retry..");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected !");
  Serial.println("esp8266 IP address: ");
  Serial.println(WiFi.localIP());
}


// MQTT Connect
void mqttConnect() {
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    String clientId = "ESP8266-";  // Create client ID
    clientId += String(random(0xffff), HEX);

    if (client.connect(clientId.c_str())) {
      Serial.println("MQTT connected");
      client.publish(MQTT_INIT_TOPIC, MQTT_INIT_TOPIC_MSG);
      client.subscribe(MQTT_LED_TOPIC);
    } else {
      Serial.print("Conecct failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }

  }
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message Recv [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();

  // Switch on the LED if an 1 was received as first character
  if ((char)payload[0] == '1') {
    for(int i=0; i<4; i=i+1) {
      digitalWrite(ledPin, HIGH);
      delay(500);
      digitalWrite(ledPin, LOW);
      delay(500);
    }
    digitalWrite(ledPin, HIGH);
    Serial.println("ledPinHIGH");
  } else {
    digitalWrite(ledPin, LOW);
    Serial.println("ledPinLOW");
  }

}


void setup() {
  pinMode(ledPin, OUTPUT);     // LEDのピンを出力に設定
  Serial.begin(115200);
  setup_wifi();
  client.setServer(MQTT_SERVER, MQTT_PORT);
  client.setCallback(callback);
  for(int i=0; i<4; i=i+1) {
    digitalWrite(ledPin, HIGH);
    delay(500);
    digitalWrite(ledPin, LOW);
    delay(500);
  }
  digitalWrite(ledPin, HIGH);
  delay(2000);
  digitalWrite(ledPin, LOW);
}

void loop() {

  if (!client.connected()) {
    mqttConnect();
  } else {
    client.loop();
  }

  delay(1000);

}

