// load modules
var https = require('https');       //LINE APIを利用するためのHTTPSクライアント
var aws = require('aws-sdk');       //S3、Pollyを利用するためのAWS SDK
var endpoint  = 'AWS IoTのエンドポイント'; //AWS IoTのエンドポイントAWS IoTの画面から確認して置く
var s3 = new aws.S3({region: 'us-east-1'});
var BUCKET_NAME = 'iot-filesv';
var DIRECTORY_PATH = 'line'
var polly = new aws.Polly({region:'us-east-1'});

exports.handler = function(event, context) {
    console.log('EVENT:', JSON.stringify(event, null, 2));
    
    //AWS IoTの接続準備
    var iotdata = new aws.IotData( { region: 'ap-northeast-1', endpoint: endpoint } );
    var params = {
        topic: '/esp8266/led/line', // required
        payload: new Buffer('1') || 'STRING_VALUE',
        qos: 0
    }
    //AWS IoTを介して５００円LEDのTopicへMQTT Publishする
    iotdata.publish(params, function(err, data) {
      if (err) console.log("AWS IoT Error:",err, err.stack); // an error occurred
      else     console.log("AWS IoT Success:",data);           // successful response
    });
 
    //LINE メッセージをう変数へ格納
    var event_data = JSON.parse(event.body);
    var reply_token = event_data.events[0].replyToken;
    var message_text = event_data.events[0].message.text;
    var source_userid = event_data.events[0].source.userId;
    
//    if (message_text == "1") {
//        
//    }

    //Polly の準備
    pollyParams = {
        OutputFormat: 'mp3',
        VoiceId: 'Mizuki',
        Text: message_text,
        TextType: 'text'
    };
    //Polly でLINEメッセージを音声化
    polly.synthesizeSpeech(pollyParams, (err, data) => {
        if (err){
            console.log("Polly Error:", err, err.stack);
        } else {
            console.log("Polly Success:" + data);
            //S3にMP3を保存
            s3LineMsgPutParams = {
                Bucket: BUCKET_NAME,
                Key: DIRECTORY_PATH + "/line_message.mp3",
                ContentType: "audio/mpeg",
                Body: data.AudioStream
            }
            s3.putObject(s3LineMsgPutParams, function(err, data) {
                if (err) {
                    console.log(err);
                    console.log('Mp3 upload failed. file=' + DIRECTORY_PATH +  "/line_message.mp3");
                } else {
                    console.log('Mp3 upload success. file=' + DIRECTORY_PATH +  "/line_message.mp3");
                }
            });
            
            //S3にLINE IDを保存
            s3LineIdPutParams = {
                Bucket: BUCKET_NAME,
                Key: DIRECTORY_PATH + "/lineid",
                Body: source_userid
            }
            s3.putObject(s3LineIdPutParams, function(err, data) {
                if (err) {
                    console.log(err);
                    context.fail('Image upload failed. file=' + DIRECTORY_PATH +  "/lineid");
                }
                context.succeed('Image upload success. file=' + DIRECTORY_PATH +  "/lineid");
            });
            
            // LINE への受け付けましたメッセージRequest Body
            var request_body = JSON.stringify({
                replyToken: reply_token,
                messages:[
                    {
                        "type":"text",
                        "text": "[" + message_text + "] > このメッセージを音声化して相手に送りました"
                    }
                ]
            });
            // LINE への受け付けましたメッセージRequest Headers
            var send_options = {
                host: 'api.line.me',
                path: '/v2/bot/message/reply',
                headers: {
                    "Content-type": "application/json; charset=UTF-8",
                    "Authorization": " Bearer " + "LINEの「 Channel Access Token」を指定"
                },
                method: 'POST'
            };
            // LINE への受け付けましたメッセージAPIリクエスト
            var req = https.request(send_options, function(res){
                res.on('request_body', function (chunk) {
                    console.log('OK: ' + res.statusCode + chunk.toString());
                });
                req.on('error', function(err) {
                console.log('ERROR: ' + err.message);
                });
            });
            req.write(request_body);
            req.end();
        }
    });
};
