#include <ESP8266WiFi.h>
#include <PubSubClient.h>

const char* PI_SSID = "Pi3-AP";
const char* PI_PASSWORD = "raspberry";
const int   MQTT_PORT = 1883;
const char* MQTT_SERVER = "10.0.0.254";
const char* MQTT_INIT_TOPIC = "/dev/init/msg/";
const char* MQTT_SN_TOPIC = "/esp8266/humansn/twitter";
const char* MQTT_CAMERA_TOPIC = "/esp8266/humansn/picamera";
const char* MQTT_INIT_TOPIC_MSG = "{\"dev\"=\"humansn\",\"msg\"=\"init\"}";

// Pin Setting
const int motionPin = 16;    // 人感センサーpin（定数）
uint8_t motionStatus = LOW;
uint8_t beforeMotionStatus = 0;

WiFiClient espClient;
PubSubClient client(espClient);

//WiFi Setting
void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(PI_SSID);

  WiFi.begin(PI_SSID, PI_PASSWORD);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("retry..");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected !");
  Serial.println("esp8266 IP address: ");
  Serial.println(WiFi.localIP());
}


// MQTT Connect
void mqttConnect() {
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    String clientId = "ESP8266-";  // Create client ID
    clientId += String(random(0xffff), HEX);

    if (client.connect(clientId.c_str())) {
      Serial.println("MQTT connected");
      client.publish(MQTT_INIT_TOPIC, MQTT_INIT_TOPIC_MSG);
    } else {
      Serial.print("Conecct failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }

  }
}

void setup() {
  pinMode(motionPin,INPUT);
  Serial.begin(115200);
  setup_wifi();
  client.setServer(MQTT_SERVER, MQTT_PORT);
}

void loop() {

  if (!client.connected()) {
    mqttConnect();
  } else {
    client.loop();
  }

  motionStatus = digitalRead(motionPin);
  if ( motionStatus == 0 ) {
    Serial.println("motion LOW");
    if ( beforeMotionStatus == 1 ) {
      client.publish(MQTT_SN_TOPIC, "{\"name\": \"humansn\", \"data\": [ {\"type\": \"humansn\", \"value\": 0 } ] }");
      Serial.println("motion LOW Publish!");
      beforeMotionStatus = 0;
    }
  }else{
    if ( beforeMotionStatus == 0 ) {
      client.publish(MQTT_SN_TOPIC, "{\"name\": \"humansn\", \"data\": [ {\"type\": \"humansn\", \"value\": 1 } ] }");
      client.publish(MQTT_CAMERA_TOPIC, "{\"capture\": \"on\"}");
      Serial.println("motion HIGH Publish!");
      beforeMotionStatus = 1;
    }
    Serial.println("motion HIGH");
    delay(5000);
  }

  delay(300);

}

