import picamera
import boto3
import paho.mqtt.client as mqtt
import json

IMG_WIDTH = 640 # max 2592
IMG_HEIGHT = 480 # max 1944
IMG_DIR = 'picamera/'
IMG_NAME = 'img.jpg'
S3_BUCKET_NAME = 'myimg01'

MQTT_HOST = '127.0.0.1'
MQTT_PORT = 1883
MQTT_TOPIC = '/esp8266/humansn/picamera'
MQTT_KEEPALIVE = 60
MQTT_QOS = 0


def on_connect(mqtt_cl, obj, res_code):
	print("mqtt connect: res_code: "+str(res_code))

def on_subscribe(mqttc, obj, mid, granted_qos):
	print("Subscrib: "+str(mid)+" "+str(granted_qos))

def on_message(mqtt_cl, obj, msg):
	print("message recv: "+msg.topic+" "+str(msg.qos)+" "+str(msg.payload))
	try:
		mqtt_data = json.loads(msg.payload)
	except ValueError as e:
		print e
		mqtt_data = json.loads('{"capture": "off"}')
	print(json.dumps(mqtt_data))
	if mqtt_data.get('capture') == 'on':
		camera.capture(IMG_DIR + IMG_NAME)
		data = open(IMG_DIR + IMG_NAME, 'rb')
		s3.Bucket(S3_BUCKET_NAME).put_object(Key=IMG_NAME, Body=data)
		print("Send pictures to S3 :"+ IMG_NAME)
	else:
		print("Skip the capture")

camera = picamera.PiCamera()
camera.resolution = (IMG_WIDTH, IMG_HEIGHT)
camera.led = True # led on

s3 = boto3.resource('s3')

mqtt_cl = mqtt.Client()
mqtt_cl.on_connect = on_connect
mqtt_cl.on_subscribe = on_subscribe
mqtt_cl.on_message = on_message
mqtt_cl.connect(MQTT_HOST, MQTT_PORT, MQTT_KEEPALIVE)
mqtt_cl.subscribe(MQTT_TOPIC, MQTT_QOS)
mqtt_cl.loop_forever()
