'use strict';

console.log('Loading function');

const aws = require('aws-sdk');

const s3 = new aws.S3({ apiVersion: '2006-03-01' });
const rekognition = new aws.Rekognition();
const sns = new aws.SNS({region: 'ap-northeast-1'});
    
exports.handler = (event, context, callback) => {
    console.log('Received event:', JSON.stringify(event, null, 2));

    // Get the object from the event and show its content type
    const bucket = event.Records[0].s3.bucket.name;
    const key = decodeURIComponent(event.Records[0].s3.object.key.replace(/\+/g, ' '));
    const params = {
        Bucket: bucket,
        Key: key,
    };
    s3.getObject(params, (err, data) => {
        if (err) {
            console.log(err);
            const message = `Error getting object ${key} from bucket ${bucket}. Make sure they exist and your bucket is in the same region as this function.`;
            console.log(message);
            callback(message);
        } else {
            console.log('CONTENT TYPE:', data.ContentType);
            console.log('data', data);
            callback(null, data.ContentType);
            const r_params = {
                CollectionId: "iot-test",
                FaceMatchThreshold: 80,
                Image: {
                    S3Object: {
                        Bucket:"myimg01",
                        Name: event.Records[0].s3.object.key
                    }
                },
                MaxFaces:5
            };
            rekognition.searchFacesByImage(r_params, function(err, r_data) {
                var smsMessage = "";
                if (err) {
                    console.log('Rekognition Error: ',err, err.stack);
                    smsMessage = '人ではないものを検知しました。';
                //else     console.log(r_data);
                } else {
                    console.log('Rekognition Success: ',JSON.stringify(r_data));
                    for(var i = 0 ; i < r_data.FaceMatches.length ; i++ ) {
                        if (r_data.FaceMatches[i].Face.ExternalImageId == 'rinta.jpg') {
                            smsMessage +=
                            'りんたろうくんを検知しました[' + r_data.FaceMatches[i].Similarity + '％]';
                        } else if(r_data.FaceMatches[i].Face.ExternalImageId == 'haru.jpg') {
                            smsMessage +=
                            'はるくんを検知しました[' + r_data.FaceMatches[i].Similarity + '％]';
                        } else {
                            smsMessage +=
                            r_data.FaceMatches[i].Face.ExternalImageId + 'を検知しました[' + r_data.FaceMatches[i].Similarity + '％]';
                        }
                    }
                    if (r_data.FaceMatches.length === 0) {
                        smsMessage += '知らない人を検知しました';
                    }
                    console.log(smsMessage);
                }
                sns.publish({
                        Message: smsMessage,
                        Subject: '人検知',
                        TopicArn: 'smsトピック名'
                    }, function(err, data){
                        if ( err ) context.fail('fail');
                    });
            });
        }
    });
};
