#include <ESP8266WiFi.h>
#include <PubSubClient.h>

const char* PI_SSID = "Pi3-AP";
const char* PI_PASSWORD = "raspberry";
const int   MQTT_PORT = 1883;
const char* MQTT_SERVER = "10.0.0.254";
const char* MQTT_INIT_TOPIC = "/dev/init/msg/";
const char* MQTT_LINEBUTTON_TOPIC = "/esp8266/button/line";       //LINE button PubTopic
const char* MQTT_LED_TOPIC = "/esp8266/led/line";                 //LINE button PubTopic
const char* MQTT_SPEAKER_TOPIC = "/raspi3/loudspeaker/line";      //LINE button PubTopic
const char* MQTT_CAMERABUTTON_TOPIC = "/esp8266/button/picamera"; //PiCamera button PubTopic

const char* MQTT_INIT_TOPIC_MSG = "{\"dev\"=\"button\",\"msg\"=\"init\"}";

// Pin Setting
const int LINEButtonPin = 14;    // Line用IoTボタンpin（定数）
const int CAMERAButtonPin = 16;    // Line用IoTボタンpin（定数）

WiFiClient espClient;
PubSubClient client(espClient);

//WiFi Setting
void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(PI_SSID);

  WiFi.begin(PI_SSID, PI_PASSWORD);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("retry..");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected !");
  Serial.println("esp8266 IP address: ");
  Serial.println(WiFi.localIP());
}


// MQTT Connect
void mqttConnect() {
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    String clientId = "ESP8266-";  // Create client ID
    clientId += String(random(0xffff), HEX);

    if (client.connect(clientId.c_str())) {
      Serial.println("MQTT connected");
      client.publish(MQTT_INIT_TOPIC, MQTT_INIT_TOPIC_MSG);
    } else {
      Serial.print("Conecct failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }

  }
}

void setup() {
  pinMode(LINEButtonPin,INPUT);
  pinMode(CAMERAButtonPin,INPUT);
  Serial.begin(115200);
  setup_wifi();
  client.setServer(MQTT_SERVER, MQTT_PORT);
}

void loop() {

  if (!client.connected()) {
    mqttConnect();
  } else {
    client.loop();
  }

  if (digitalRead(LINEButtonPin) == HIGH ) {
    client.publish(MQTT_LINEBUTTON_TOPIC, "{\"msg\":\"既読しました\"}");
    client.publish(MQTT_LED_TOPIC       , "0");
    client.publish(MQTT_SPEAKER_TOPIC   , "{\"S3_BUCKET_NAME\": \"iot-filesv\",\"S3_KEY\": \"line/line_message.mp3\"}");
    Serial.print("Publish: ");
    Serial.println(MQTT_LINEBUTTON_TOPIC);
  }
  if (digitalRead(CAMERAButtonPin) == HIGH ) {
    client.publish(MQTT_CAMERABUTTON_TOPIC, "{\"capture\": \"on\"}");
    Serial.print("Publish: ");
    Serial.println(MQTT_CAMERABUTTON_TOPIC);
  }

  delay(300);
}

