#include <ESP8266WiFi.h>
#include <PubSubClient.h>

const char* PI_SSID = "Pi3-AP";
const char* PI_PASSWORD = "raspberry";
const int   MQTT_PORT = 1883;
const char* MQTT_SERVER = "10.0.0.254";
const char* MQTT_INIT_TOPIC = "/dev/init/msg/";
const char* MQTT_DOOR_TOPIC = "/esp8266/door/twilio";
const char* MQTT_INIT_TOPIC_MSG = "{\"dev\"=\"humansn\",\"msg\"=\"init\"}";

// Pin Setting
const int doorPin = 14;    // 人感センサーpin（定数）
uint8_t doorStatus = LOW;
uint8_t beforeDoorStatus = 0;

WiFiClient espClient;
PubSubClient client(espClient);

//WiFi Setting
void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(PI_SSID);

  WiFi.begin(PI_SSID, PI_PASSWORD);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("retry..");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected !");
  Serial.println("esp8266 IP address: ");
  Serial.println(WiFi.localIP());
}


// MQTT Connect
void mqttConnect() {
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    String clientId = "ESP8266-";  // Create client ID
    clientId += String(random(0xffff), HEX);

    if (client.connect(clientId.c_str())) {
      Serial.println("MQTT connected");
      client.publish(MQTT_INIT_TOPIC, MQTT_INIT_TOPIC_MSG);
    } else {
      Serial.print("Conecct failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      delay(5000);
    }

  }
}

void setup() {
  pinMode(doorPin,INPUT_PULLUP);
  Serial.begin(115200);
  setup_wifi();
  client.setServer(MQTT_SERVER, MQTT_PORT);
}

void loop() {

  if (!client.connected()) {
    mqttConnect();
  } else {
    client.loop();
  }

  doorStatus = digitalRead(doorPin);
  Serial.println("door Status:" + doorStatus);
  if ( doorStatus == 0 ) {
    Serial.println("door LOW");
    if ( beforeDoorStatus == 1 ) {
      client.publish(MQTT_DOOR_TOPIC, "{\"name\": \"door\", \"data\": 0 }");
      Serial.println("door LOW Publish!");
      beforeDoorStatus = 0;
    }
  }else{
    if ( beforeDoorStatus == 0 ) {
      client.publish(MQTT_DOOR_TOPIC, "{\"name\": \"door\", \"data\": 1 }");
      Serial.println("door HIGH Publish!");
      beforeDoorStatus = 1;
    }
    Serial.println("door HIGH");
    delay(5000);
  }

  delay(1000);

}

